'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.buildSnapshotResolver = exports.isSnapshotPath = exports.DOT_EXTENSION = exports.EXTENSION = void 0;

var _chalk = _interopRequireDefault(require('chalk'));

var _path = _interopRequireDefault(require('path'));

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}

// Copyright (c) 2014-present, Facebook, Inc. All rights reserved.
const EXTENSION = 'snap';
exports.EXTENSION = EXTENSION;
const DOT_EXTENSION = '.' + EXTENSION;
exports.DOT_EXTENSION = DOT_EXTENSION;

const isSnapshotPath = path => path.endsWith(DOT_EXTENSION);

exports.isSnapshotPath = isSnapshotPath;
const cache = new Map();

const buildSnapshotResolver = config => {
  const key = config.rootDir;

  if (!cache.has(key)) {
    cache.set(key, createSnapshotResolver(config.snapshotResolver));
  }

  return cache.get(key);
};

exports.buildSnapshotResolver = buildSnapshotResolver;

function createSnapshotResolver(snapshotResolverPath) {
  return typeof snapshotResolverPath === 'string'
    ? createCustomSnapshotResolver(snapshotResolverPath)
    : {
        resolveSnapshotPath: testPath =>
          _path.default.join(
            _path.default.join(
              _path.default.dirname(testPath),
              '__snapshots__'
            ),
            _path.default.basename(testPath) + DOT_EXTENSION
          ),
        resolveTestPath: snapshotPath =>
          _path.default.resolve(
            _path.default.dirname(snapshotPath),
            '..',
            _path.default.basename(snapshotPath, DOT_EXTENSION)
          )
      };
}

function createCustomSnapshotResolver(snapshotResolverPath) {
  const custom = require(snapshotResolverPath);

  if (typeof custom.resolveSnapshotPath !== 'function') {
    throw new TypeError(mustImplement('resolveSnapshotPath'));
  }

  if (typeof custom.resolveTestPath !== 'function') {
    throw new TypeError(mustImplement('resolveTestPath'));
  }

  const customResolver = {
    resolveSnapshotPath: testPath =>
      custom.resolveSnapshotPath(testPath, DOT_EXTENSION),
    resolveTestPath: snapshotPath =>
      custom.resolveTestPath(snapshotPath, DOT_EXTENSION)
  };
  verifyConsistentTransformations(customResolver);
  return customResolver;
}

function mustImplement(functionName) {
  return (
    _chalk.default.bold(
      `Custom snapshot resolver must implement a \`${functionName}\` function.`
    ) +
    '\nDocumentation: https://facebook.github.io/jest/docs/en/configuration.html#snapshotResolver'
  );
}

function verifyConsistentTransformations(custom) {
  const fakeTestPath = _path.default.posix.join(
    'some-path',
    '__tests__',
    'snapshot_resolver.test.js'
  );

  const transformedPath = custom.resolveTestPath(
    custom.resolveSnapshotPath(fakeTestPath)
  );

  if (transformedPath !== fakeTestPath) {
    throw new Error(
      _chalk.default.bold(
        `Custom snapshot resolver functions must transform paths consistently, i.e. expects resolveTestPath(resolveSnapshotPath('${fakeTestPath}')) === ${transformedPath}`
      )
    );
  }
}
